package tech.sbdevelopment.v10lift.commands;

import co.aikar.commands.BaseCommand;
import co.aikar.commands.CommandHelp;
import co.aikar.commands.annotation.*;
import com.cryptomorin.xseries.XMaterial;
import org.bukkit.*;
import org.bukkit.block.Block;
import org.bukkit.block.BlockState;
import org.bukkit.block.Sign;
import org.bukkit.block.data.BlockData;
import org.bukkit.block.data.Powerable;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import tech.sbdevelopment.v10lift.V10LiftPlugin;
import tech.sbdevelopment.v10lift.api.V10LiftAPI;
import tech.sbdevelopment.v10lift.api.objects.*;
import tech.sbdevelopment.v10lift.managers.AntiCopyBlockManager;
import tech.sbdevelopment.v10lift.managers.DataManager;
import tech.sbdevelopment.v10lift.managers.ForbiddenBlockManager;
import tech.sbdevelopment.v10lift.managers.VaultManager;
import tech.sbdevelopment.v10lift.sbutils.LocationSerializer;
import tech.sbdevelopment.v10lift.utils.ConfigUtil;
import tech.sbdevelopment.v10lift.utils.WorldEditUtil;

import javax.annotation.Nonnull;
import java.util.Optional;
import java.util.*;

@CommandAlias("v10lift|v10l")
@Description("The main command for V10Lift")
public class V10LiftCommand extends BaseCommand {
    @HelpCommand
    @CatchUnknown
    @Default
    public static void onHelp(CommandSender sender, CommandHelp help) {
        help.showHelp();
    }

    @Subcommand("info")
    @Description("Get information about the plugin")
    public void infoCommand(CommandSender sender) {
        sender.sendMessage(ChatColor.DARK_GREEN + "V10Lift v" + V10LiftPlugin.getInstance().getDescription().getVersion());
        sender.sendMessage(ChatColor.GREEN + "Made by SBDeveloper");
        sender.sendMessage(ChatColor.GREEN + " ");
        sender.sendMessage(ChatColor.GREEN + "Type /v10lift help for more information!");
    }

    @Subcommand("list")
    @Description("List all lifts")
    @CommandPermission("v10lift.list")
    public void listCommand(CommandSender sender) {
        //Send list of all lifts
        List<Lift> lifts = V10LiftPlugin.getApi().getLifts();
        if (lifts.isEmpty()) {
            ConfigUtil.sendMessage(sender, "List.NoLifts");
            return;
        }
        ConfigUtil.sendMessage(sender, "List.Header");
        for (Lift lift : lifts) {
            ConfigUtil.sendMessage(sender, "List.Lift", Map.of("%Name%", lift.getName()));
        }
    }

    @Subcommand("create")
    @Description("Create a lift")
    @CommandPermission("v10lift.admin")
    @CommandCompletion("@nothing")
    public void createCommand(Player p, @co.aikar.commands.annotation.Optional String name) {
        if (DataManager.containsPlayer(p.getUniqueId())) {
            //Already building!!
            if (name == null) {
                ConfigUtil.sendMessage(p, "General.IncorrectUsage", Collections.singletonMap("%Command%", "/v10lift create <Name>"));
                return;
            }

            TreeSet<LiftBlock> blocks = DataManager.getPlayer(p.getUniqueId());
            if (blocks.isEmpty()) {
                ConfigUtil.sendMessage(p, "Create.NoBlocks");
                return;
            }

            Lift lift = V10LiftPlugin.getApi().createLift(p, name);
            if (lift == null) {
                ConfigUtil.sendMessage(p, "General.AlreadyExists");
                return;
            }

            blocks.forEach(block -> lift.addBlock(lift.getBlocks(), block));
            lift.sortLiftBlocks();
            DataManager.removePlayer(p.getUniqueId());
            ConfigUtil.sendMessage(p, "Create.Created", Collections.singletonMap("%Name%", name));
            p.performCommand("v10lift edit " + name);
        } else {
            //Not building yet!!
            DataManager.addPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(p, "Create.AddBlocks");
        }
    }

    @Subcommand("delete")
    @Description("Delete a lift")
    @CommandPermission("v10lift.admin")
    @CommandCompletion("@lifts")
    public void deleteCommand(Player p, Lift lift) {
        if (!V10LiftPlugin.getApi().deleteLift(lift)) {
            ConfigUtil.sendMessage(p, "Delete.NotRemoved", Collections.singletonMap("%Name%", lift.getName()));
            return;
        }

        ConfigUtil.sendMessage(p, "Delete.Removed", Collections.singletonMap("%Name%", lift.getName()));
    }

    private boolean setOfflineCommand(CommandSender sender, @Nonnull String[] args) {
        String liftName = args[1];
        boolean newState = Boolean.parseBoolean(args[2]);
        Optional<Lift> liftOpt = V10LiftPlugin.getApi().getLift(liftName);
        if (liftOpt.isEmpty()) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }
        Lift lift = liftOpt.get();

        lift.setOffline(newState);

        //Update all offline inputs
        for (LiftInput li : lift.getOfflineInputs()) {
            Block b = Bukkit.getWorld(li.getWorld()).getBlockAt(li.getX(), li.getY(), li.getZ());
            BlockData bd = b.getBlockData();
            if (!(bd instanceof Powerable)) continue;
            ((Powerable) bd).setPowered(newState);
            b.setBlockData(bd);
        }

        if (lift.isOffline()) {
            ConfigUtil.sendMessage(sender, "SetOffline.Disabled");
        } else {
            ConfigUtil.sendMessage(sender, "SetOffline.Enabled");
        }
        return true;
    }

    private boolean disableCommand(CommandSender sender, @Nonnull String[] args) {
        String liftName = args[1];
        Optional<Lift> liftOpt = V10LiftPlugin.getApi().getLift(liftName);
        if (liftOpt.isEmpty()) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }
        Lift lift = liftOpt.get();

        if (lift.isDefective()) {
            ConfigUtil.sendMessage(sender, "Disable.AlreadyDefective");
            return true;
        }

        lift.setDefective(true);
        ConfigUtil.sendMessage(sender, "Disable.Disabled");
        return true;
    }

    private boolean stopCommand(CommandSender sender, @Nonnull String[] args) {
        Optional<Lift> liftOpt;
        if (args.length == 1 && sender instanceof Player) {
            //v10lift stop -> Get liftName from loc and floorName from sign
            Player p = (Player) sender;
            liftOpt = V10LiftPlugin.getApi().getLift(p.getLocation());
        } else if (args.length == 1) {
            ConfigUtil.sendMessage(sender, "Stop.NonPlayer");
            return true;
        } else {
            liftOpt = V10LiftPlugin.getApi().getLift(args[1]);
        }

        if (liftOpt.isEmpty()) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }
        Lift lift = liftOpt.get();

        if (lift.getQueue().hasRequests()) lift.getQueue().empty();

        if (!DataManager.containsMovingTask(lift.getName())) {
            ConfigUtil.sendMessage(sender, "Stop.NoMovingTasks", Collections.singletonMap("%Name%", lift.getName()));
            return true;
        }

        Bukkit.getScheduler().cancelTask(DataManager.getMovingTask(lift.getName()));
        DataManager.removeMovingTask(lift.getName());
        ConfigUtil.sendMessage(sender, "Stop.Stopped", Collections.singletonMap("%Name%", lift.getName()));
        return true;
    }

    private boolean startCommand(CommandSender sender, @Nonnull String[] args) {
        Optional<Lift> liftOpt;
        if (args.length == 1 && sender instanceof Player) {
            //v10lift stop -> Get liftName from loc and floorName from sign
            Player p = (Player) sender;
            liftOpt = V10LiftPlugin.getApi().getLift(p.getLocation());
        } else if (args.length == 1) {
            ConfigUtil.sendMessage(sender, "Start.NonPlayer");
            return true;
        } else {
            liftOpt = V10LiftPlugin.getApi().getLift(args[1]);
        }

        if (liftOpt.isEmpty()) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }
        Lift lift = liftOpt.get();

        String floorName = null;
        if (args.length == 1 || args.length == 2) {
            for (LiftBlock lb : lift.getBlocks()) {
                Block block = Objects.requireNonNull(Bukkit.getWorld(lb.getWorld()), "World is null at start command").getBlockAt(lb.getX(), lb.getY(), lb.getZ());
                if (block.getState() instanceof Sign) {
                    Sign sign = (Sign) block.getState();
                    String f = ChatColor.stripColor(sign.getLine(3));
                    if (lift.getFloors().containsKey(f)) {
                        floorName = f;
                    }
                }
            }
        } else {
            floorName = args[2];
        }

        if (floorName == null || !lift.getFloors().containsKey(floorName)) {
            if (floorName == null) floorName = "null";
            ConfigUtil.sendMessage(sender, "Start.FloorDoesntExists", Collections.singletonMap("%Name%", floorName));
            return true;
        }

        Floor f = lift.getFloors().get(floorName);
        lift.getQueue().requestFloor(floorName, f);
        ConfigUtil.sendMessage(sender, "Start.Started", Collections.singletonMap("%Name%", lift.getName()));
        return true;
    }

    private boolean reloadCommand(CommandSender sender) {
        for (Lift lift : V10LiftPlugin.getApi().getLifts()) {
            if (DataManager.containsMovingTask(lift.getName())) {
                Bukkit.getScheduler().cancelTask(DataManager.getMovingTask(lift.getName()));
            }

            lift.getQueue().empty();
            lift.sortLiftBlocks();
        }

        DataManager.clearMovingTasks();
        V10LiftPlugin.getSConfig().reloadConfig();
        try {
            V10LiftPlugin.getDBManager().save();
            V10LiftPlugin.getDBManager().load();
        } catch (Exception e) {
            e.printStackTrace();
        }

        AntiCopyBlockManager.reinit();
        ForbiddenBlockManager.reinit();

        ConfigUtil.sendMessage(sender, "Reload.Reloaded");
        return true;
    }

    private boolean abortCommand(CommandSender sender) {
        Player p = (Player) sender;
        boolean abort = false;

        if (DataManager.containsPlayer(p.getUniqueId())) {
            DataManager.removePlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsWhoisREQPlayer(p.getUniqueId())) {
            DataManager.removeWhoisREQPlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsInputEditsPlayer(p.getUniqueId())) {
            DataManager.removeInputEditsPlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsInputRemovesPlayer(p.getUniqueId())) {
            DataManager.removeInputRemovesPlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsOfflineEditsPlayer(p.getUniqueId())) {
            DataManager.removeOfflineEditsPlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsOfflineRemovesPlayer(p.getUniqueId())) {
            DataManager.removeOfflineRemovesPlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsBuilderPlayer(p.getUniqueId())) {
            DataManager.removeBuilderPlayer(p.getUniqueId());
            V10LiftPlugin.getApi().getLift(DataManager.getEditPlayer(p.getUniqueId())).get().sortLiftBlocks();
            abort = true;
        }

        if (DataManager.containsRopeEditPlayer(p.getUniqueId())) {
            DataManager.removeRopeEditPlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsRopeRemovesPlayer(p.getUniqueId())) {
            DataManager.removeRopeRemovesPlayer(p.getUniqueId());
            abort = true;
        }

        if (DataManager.containsDoorEditPlayer(p.getUniqueId())) {
            DataManager.removeDoorEditPlayer(p.getUniqueId());
            abort = true;
        }

        if (abort) {
            ConfigUtil.sendMessage(sender, "Abort.Cancelled");
        } else {
            ConfigUtil.sendMessage(sender, "Abort.NothingToCancel");
        }
        return true;
    }

    private boolean repairCommand(CommandSender sender, @Nonnull String[] args) {
        String liftName = args[1];
        Optional<Lift> liftOpt = V10LiftPlugin.getApi().getLift(liftName);
        if (liftOpt.isEmpty()) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }
        Lift lift = liftOpt.get();

        if (!lift.isDefective()) {
            ConfigUtil.sendMessage(sender, "Repair.NotDefective");
            return true;
        }

        if (sender instanceof Player) {
            Player p = (Player) sender;
            if (!p.hasPermission("v10lift.admin")) {
                int masterAmount = V10LiftPlugin.getSConfig().getFile().getInt("MasterRepairAmount");
                Optional<XMaterial> mat = XMaterial.matchXMaterial(Objects.requireNonNull(V10LiftPlugin.getSConfig().getFile().getString("MasterRepairItem"), "MasterRepairItem is null"));
                if (!mat.isPresent()) {
                    Bukkit.getLogger().severe("[V10Lift] The material for MasterRepairItem is undefined!");
                    return true;
                }
                Material masterItem = mat.get().parseMaterial();
                if (masterItem == null) {
                    Bukkit.getLogger().severe("[V10Lift] The material for MasterRepairItem is undefined!");
                    return true;
                }
                if (p.getGameMode() != GameMode.CREATIVE && masterAmount > 0) {
                    if (!p.getInventory().contains(masterItem)) {
                        Map<String, String> replacements = new HashMap<>();
                        replacements.put("%Amount%", String.valueOf(masterAmount));
                        replacements.put("%ItemName%", masterItem.toString().toLowerCase());
                        ConfigUtil.sendMessage(sender, "Repair.ItemsNeeded", replacements);
                        return true;
                    }
                    p.getInventory().remove(new ItemStack(masterItem, masterAmount));
                }
            }
        }
        lift.setDefective(false);
        ConfigUtil.sendMessage(sender, "Repair.Repaired");
        return true;
    }

    private boolean realisticCommand(CommandSender sender) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        Lift lift = V10LiftPlugin.getApi().getLift(DataManager.getEditPlayer(p.getUniqueId())).get();
        lift.setRealistic(!lift.isRealistic());
        if (lift.isRealistic()) {
            ConfigUtil.sendMessage(sender, "Realistic.TurnedOn");
        } else {
            ConfigUtil.sendMessage(sender, "Realistic.TurnedOff");
        }
        return true;
    }

    private boolean soundCommand(CommandSender sender) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        Lift lift = V10LiftPlugin.getApi().getLift(DataManager.getEditPlayer(p.getUniqueId())).get();
        lift.setSound(!lift.isSound());
        if (lift.isSound()) {
            ConfigUtil.sendMessage(sender, "Sound.TurnedOn");
        } else {
            ConfigUtil.sendMessage(sender, "Sound.TurnedOff");
        }
        return true;
    }

    private boolean speedCommand(CommandSender sender, @Nonnull String[] args) {
        Lift lift;
        if (args.length == 3) {
            if (!V10LiftPlugin.getApi().isLift(args[2])) {
                ConfigUtil.sendMessage(sender, "General.DoesntExists");
                return true;
            }
            lift = V10LiftPlugin.getApi().getLift(args[2]).get();
        } else {
            Player p = (Player) sender;
            if (!DataManager.containsEditPlayer(p.getUniqueId())) {
                ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
                return true;
            }

            lift = V10LiftPlugin.getApi().getLift(DataManager.getEditPlayer(p.getUniqueId())).get();
        }

        try {
            int speed = Integer.parseInt(args[1]);
            lift.setSpeed(speed);
            if (lift.getSpeed() < 1) lift.setSpeed(1);
            ConfigUtil.sendMessage(sender, "Speed.Changed");
        } catch (NumberFormatException e) {
            ConfigUtil.sendMessage(sender, "Speed.WrongSpeed", Collections.singletonMap("%Speed%", args[1]));
        }
        return true;
    }

    private boolean whoisCommand(CommandSender sender, @Nonnull String[] args) {
        if (args.length < 2) {
            if (!(sender instanceof Player)) {
                ConfigUtil.sendMessage(sender, "Whois.UseWithoutName");
                return true;
            }

            //Without name
            Player p = (Player) sender;
            DataManager.addWhoisREQPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(sender, "Whois.WithoutName");
        } else {
            String liftName = args[1];
            if (!V10LiftPlugin.getApi().isLift(liftName)) {
                ConfigUtil.sendMessage(sender, "Whois.DoesntExists");
            } else {
                V10LiftPlugin.getApi().getLift(liftName).get().sendInfo(sender);
            }
        }
        return true;
    }

    private boolean whitelistCommand(CommandSender sender, @Nonnull String[] args) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        Lift lift = DataManager.getLift(DataManager.getEditPlayer(p.getUniqueId()));
        boolean isGroup = false;
        String wgn = null;
        UUID wpu = null;
        if (args[2].startsWith("g:")) {
            if (!V10LiftPlugin.isVaultEnabled()) {
                ConfigUtil.sendMessage(sender, "Whitelist.Group.VaultNotFound");
                return true;
            }
            isGroup = true;
            wgn = args[2].replace("g:", "");
            if (!VaultManager.isGroup(wgn)) {
                ConfigUtil.sendMessage(sender, "Whitelist.Group.NotFound", Collections.singletonMap("%Name%", wgn));
                return true;
            }
        } else {
            OfflinePlayer wp = Bukkit.getOfflinePlayer(args[2]);
            wpu = wp.getUniqueId();
            if (!wp.hasPlayedBefore()) {
                ConfigUtil.sendMessage(sender, "Whitelist.Player.NotFound", Collections.singletonMap("%Name%", args[2]));
                return true;
            }
        }

        String floor = null;
        if (args.length < 4) {
            Block b = p.getLocation().getBlock();
            Floor f = new Floor(b.getY() - 1, Objects.requireNonNull(b.getWorld(), "World was null at whitelistCommand").getName());
            if (!lift.getFloors().containsValue(f)) {
                ConfigUtil.sendMessage(sender, "General.DetectionFailed");
                return true;
            }
            for (Map.Entry<String, Floor> e : lift.getFloors().entrySet()) {
                Floor fl = e.getValue();
                if (fl.equals(f)) {
                    floor = e.getKey();
                    break;
                }
            }
        } else {
            floor = args[3];
            if (!lift.getFloors().containsKey(floor)) {
                ConfigUtil.sendMessage(sender, "General.FloorDoesntExists", Collections.singletonMap("%Name%", args[3]));
                return true;
            }
        }

        Floor f = lift.getFloors().get(floor);
        if (args[1].equalsIgnoreCase("add")) {
            if (isGroup) {
                if (f.getGroupWhitelist().contains(wgn)) {
                    ConfigUtil.sendMessage(sender, "Whitelist.Group.AlreadyContains");
                } else {
                    f.getGroupWhitelist().add(wgn);
                    ConfigUtil.sendMessage(sender, "Whitelist.Player.Added");
                }
            } else {
                if (f.getUserWhitelist().contains(wpu)) {
                    ConfigUtil.sendMessage(sender, "Whitelist.Group.AlreadyContains");
                } else {
                    f.getUserWhitelist().add(wpu);
                    ConfigUtil.sendMessage(sender, "Whitelist.Player.Added");
                }
            }
        } else if (args[1].equalsIgnoreCase("del")) {
            if (isGroup) {
                if (!f.getGroupWhitelist().contains(wgn)) {
                    ConfigUtil.sendMessage(sender, "Whitelist.Group.DoesntContains");
                } else {
                    f.getGroupWhitelist().remove(wgn);
                    ConfigUtil.sendMessage(sender, "Whitelist.Group.Removed");
                }
            } else {
                if (!f.getUserWhitelist().contains(wpu)) {
                    ConfigUtil.sendMessage(sender, "Whitelist.Player.DoesntContains");
                } else {
                    f.getUserWhitelist().remove(wpu);
                    ConfigUtil.sendMessage(sender, "Whitelist.User.Removed");
                }
            }
        } else {
            return helpCommand(sender);
        }
        return true;
    }

    private boolean doorCommand(CommandSender sender, String[] args) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        if (DataManager.containsDoorEditPlayer(p.getUniqueId())) {
            DataManager.removeDoorEditPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(sender, "Door.Disabled");
            return true;
        }

        Lift lift = DataManager.getLift(DataManager.getEditPlayer(p.getUniqueId()));
        String floor = null;
        if (args.length < 2) {
            Location loc = p.getLocation();
            Floor f = new Floor(loc.getBlockY() - 1, Objects.requireNonNull(loc.getWorld(), "World was null at doorCommand").getName());
            if (!lift.getFloors().containsValue(f)) {
                ConfigUtil.sendMessage(sender, "General.DetectionFailed");
                return true;
            }
            for (Map.Entry<String, Floor> e : lift.getFloors().entrySet()) {
                Floor fl = e.getValue();
                if (fl.equals(f)) {
                    floor = e.getKey();
                    break;
                }
            }
        } else {
            floor = args[2];
            if (!lift.getFloors().containsKey(floor)) {
                ConfigUtil.sendMessage(sender, "General.FloorDoesntExists", Collections.singletonMap("%Name", args[2]));
                return true;
            }
        }
        DataManager.addDoorEditPlayer(p.getUniqueId(), floor);
        ConfigUtil.sendMessage(sender, "Door.Enabled");
        return true;
    }

    private boolean ropeCommand(CommandSender sender, String[] args) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        if (args[1].equalsIgnoreCase("add")) {
            if (DataManager.containsRopeEditPlayer(p.getUniqueId()) || DataManager.containsRopeRemovesPlayer(p.getUniqueId())) {
                ConfigUtil.sendMessage(sender, "Rope.StillAdjusting");
                return true;
            }
            DataManager.addRopeEditPlayer(p.getUniqueId(), null);
            ConfigUtil.sendMessage(sender, "Rope.Add");
        } else if (args[1].equalsIgnoreCase("del")) {
            if (DataManager.containsRopeEditPlayer(p.getUniqueId()) || DataManager.containsRopeRemovesPlayer(p.getUniqueId())) {
                ConfigUtil.sendMessage(sender, "Rope.StillAdjusting");
                return true;
            }
            DataManager.addRopeRemovesPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(sender, "Rope.Delete");
        }
        return true;
    }

    private boolean buildWorldEditCommand(CommandSender sender) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        if (!DataManager.containsBuilderPlayer(p.getUniqueId())) {
            DataManager.addBuilderPlayer(p.getUniqueId());
        }

        Integer blocks = WorldEditUtil.getBlocksInSelection(p);
        if (blocks == -1) {
            ConfigUtil.sendMessage(sender, "Build.NoSelection");
        } else if (blocks == -2) {
            ConfigUtil.sendMessage(sender, "Build.UnsupportedSelection");
        } else if (blocks == 0) {
            ConfigUtil.sendMessage(sender, "Build.BlocksAdded");
        } else {
            ConfigUtil.sendMessage(sender, "Build.BlocksFailed", Collections.singletonMap("%Failed%", String.valueOf(blocks)));
        }

        if (DataManager.containsBuilderPlayer(p.getUniqueId())) {
            DataManager.removeBuilderPlayer(p.getUniqueId());
            V10LiftAPI.getInstance().sortLiftBlocks(DataManager.getEditPlayer(p.getUniqueId()));
            ConfigUtil.sendMessage(sender, "Build.Disabled");
        }

        return true;
    }

    private boolean buildCommand(CommandSender sender) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        if (DataManager.containsBuilderPlayer(p.getUniqueId())) {
            DataManager.removeBuilderPlayer(p.getUniqueId());
            V10LiftAPI.getInstance().sortLiftBlocks(DataManager.getEditPlayer(p.getUniqueId()));
            ConfigUtil.sendMessage(sender, "Build.Disabled");
        } else {
            DataManager.addBuilderPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(sender, "Build.Enabled");
        }
        return true;
    }

    private boolean renameCommand(CommandSender sender, String[] args) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        String liftName = DataManager.getEditPlayer(p.getUniqueId());
        if (!DataManager.containsLift(liftName)) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }

        Bukkit.dispatchCommand(sender, "v10lift edit");
        V10LiftAPI.getInstance().renameLift(liftName, args[1]);
        Bukkit.dispatchCommand(sender, "v10lift edit " + args[1]);

        ConfigUtil.sendMessage(sender, "Rename.Renamed");
        return true;
    }

    private boolean offlineCommand(CommandSender sender, String[] args) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        String liftName = DataManager.getEditPlayer(p.getUniqueId());
        if (!DataManager.containsLift(liftName)) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }

        Lift lift = DataManager.getLift(liftName);
        if (args[1].equalsIgnoreCase("add")) {
            if (DataManager.containsOfflineEditsPlayer(p.getUniqueId()) || DataManager.containsOfflineRemovesPlayer(p.getUniqueId())) {
                ConfigUtil.sendMessage(sender, "OfflineInput.StillAdjusting");
                return true;
            }

            DataManager.addOfflineEditsPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(sender, "OfflineInput.RightClick");
        } else if (args[1].equalsIgnoreCase("del")) {
            if (lift.getOfflineInputs().isEmpty()) {
                ConfigUtil.sendMessage(sender, "OfflineInput.NothingToRemove");
                return true;
            }

            if (DataManager.containsOfflineEditsPlayer(p.getUniqueId()) || DataManager.containsOfflineRemovesPlayer(p.getUniqueId())) {
                ConfigUtil.sendMessage(sender, "OfflineInput.StillAdjusting");
                return true;
            }

            DataManager.addOfflineRemovesPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(sender, "OfflineInput.RightClick");
        } else {
            return helpCommand(sender);
        }
        return true;
    }

    private boolean inputCommand(CommandSender sender, String[] args) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        String liftName = DataManager.getEditPlayer(p.getUniqueId());
        if (!DataManager.containsLift(liftName)) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }

        Lift lift = DataManager.getLift(liftName);
        if (args[1].equalsIgnoreCase("add")) {
            String floor = null;
            if (args.length < 3) {
                Block b = p.getLocation().getBlock();
                Floor f = new Floor(b.getY() - 1, b.getWorld().getName());
                if (!lift.getFloors().containsValue(f)) {
                    ConfigUtil.sendMessage(sender, "General.DetectionFailed");
                    return true;
                }

                for (Map.Entry<String, Floor> e : lift.getFloors().entrySet()) {
                    Floor fl = e.getValue();
                    if (fl.equals(f)) {
                        floor = e.getKey();
                    }
                }
            } else {
                floor = args[2];
            }

            if (DataManager.containsInputEditsPlayer(p.getUniqueId()) || DataManager.containsInputRemovesPlayer(p.getUniqueId())) {
                ConfigUtil.sendMessage(sender, "Input.StillAdjusting");
                return true;
            }

            DataManager.addInputEditsPlayer(p.getUniqueId(), Objects.requireNonNull(floor, "Floor is null at input add command"));
            ConfigUtil.sendMessage(sender, "Input.RightClick");
        } else if (args[1].equalsIgnoreCase("del")) {
            if (lift.getInputs().isEmpty()) {
                ConfigUtil.sendMessage(sender, "Input.NothingToRemove");
                return true;
            }

            if (DataManager.containsInputEditsPlayer(p.getUniqueId()) || DataManager.containsInputRemovesPlayer(p.getUniqueId())) {
                ConfigUtil.sendMessage(sender, "Input.StillAdjusting");
                return true;
            }

            DataManager.addInputRemovesPlayer(p.getUniqueId());
            ConfigUtil.sendMessage(sender, "Input.RightClick");
        } else {
            return helpCommand(sender);
        }
        return true;
    }

    private boolean floorCommand(CommandSender sender, String[] args) {
        Player p = (Player) sender;
        if (!DataManager.containsEditPlayer(p.getUniqueId())) {
            ConfigUtil.sendMessage(sender, "General.SwitchOnEdit");
            return true;
        }

        String liftName = DataManager.getEditPlayer(p.getUniqueId());
        if (!DataManager.containsLift(liftName)) {
            ConfigUtil.sendMessage(sender, "General.DoesntExists");
            return true;
        }
        if (args[1].equalsIgnoreCase("add")) {
            Block b = p.getLocation().getBlock();
            String floorName = args[2];
            int response = V10LiftAPI.getInstance().addFloor(liftName, floorName, new Floor(b.getY() - 1, b.getWorld().getName()));
            switch (response) {
                case 0:
                    ConfigUtil.sendMessage(sender, "Floor.Added");
                    break;
                case -2:
                    ConfigUtil.sendMessage(sender, "Floor.ToHigh");
                    break;
                case -3:
                    ConfigUtil.sendMessage(sender, "Floor.AlreadyExists");
                    break;
                default:
                    ConfigUtil.sendMessage(sender, "General.InternalError");
                    break;
            }
        } else if (args[1].equalsIgnoreCase("del")) {
            String floorName = args[2];
            if (!V10LiftAPI.getInstance().removeFloor(liftName, floorName)) {
                ConfigUtil.sendMessage(sender, "General.InternalError");
            } else {
                ConfigUtil.sendMessage(sender, "Floor.Removed");
            }
        } else if (args[1].equalsIgnoreCase("rename")) {
            if (args.length < 4) {
                ConfigUtil.sendMessage(sender, "General.IncorrectUsage", Collections.singletonMap("%Command%", "/v10lift floor rename <Old name> <New name>"));
                return true;
            }

            String floorName = args[2];
            String newFloorName = args[3];
            int response = V10LiftAPI.getInstance().renameFloor(liftName, floorName, newFloorName);
            switch (response) {
                case 0:
                    ConfigUtil.sendMessage(sender, "Floor.Renamed");
                    break;
                case -2:
                    ConfigUtil.sendMessage(sender, "Floor.DoesntExists");
                    break;
                case -3:
                    ConfigUtil.sendMessage(sender, "Floor.AlreadyExists");
                    break;
                default:
                    ConfigUtil.sendMessage(sender, "General.InternalError");
                    break;
            }
        } else {
            //args[1] not found!
            return helpCommand(sender);
        }
        return true;
    }

    private boolean editCommand(@Nonnull CommandSender sender, @Nonnull String[] args) {
        Player p = (Player) sender;
        if (DataManager.containsEditPlayer(p.getUniqueId())) {
            //TURN OFF
            if (args.length < 2) {
                String liftName = DataManager.getEditPlayer(p.getUniqueId());
                if (!DataManager.containsLift(liftName)) {
                    ConfigUtil.sendMessage(sender, "General.DoesntExists");
                    return true;
                }

                Lift lift = DataManager.getLift(liftName);
                DataManager.removeEditPlayer(p.getUniqueId());
                DataManager.removeInputEditsPlayer(p.getUniqueId());
                DataManager.removeInputRemovesPlayer(p.getUniqueId());
                DataManager.removeOfflineEditsPlayer(p.getUniqueId());
                DataManager.removeOfflineRemovesPlayer(p.getUniqueId());
                if (DataManager.containsBuilderPlayer(p.getUniqueId())) {
                    DataManager.removeBuilderPlayer(p.getUniqueId());
                    V10LiftAPI.getInstance().sortLiftBlocks(liftName);
                }
                DataManager.removeRopeEditPlayer(p.getUniqueId());
                DataManager.removeRopeRemovesPlayer(p.getUniqueId());
                DataManager.removeDoorEditPlayer(p.getUniqueId());

                V10LiftPlugin.getDBManager().saveLift(liftName, lift, false);

                BlockState bs;
                Sign sign;
                for (LiftBlock lb : lift.getBlocks()) {
                    bs = Objects.requireNonNull(Bukkit.getWorld(lb.getWorld()), "World is null at edit command").getBlockAt(lb.getX(), lb.getY(), lb.getZ()).getState();
                    if (!(bs instanceof Sign)) continue;
                    sign = (Sign) bs;
                    if (!sign.getLine(0).equalsIgnoreCase(ConfigUtil.getConfigText("SignText"))) continue;
                    sign.setLine(3, "");
                    sign.update();
                }

                Iterator<LiftSign> liter = lift.getSigns().iterator();
                while (liter.hasNext()) {
                    LiftSign ls = liter.next();
                    bs = Objects.requireNonNull(Bukkit.getWorld(ls.getWorld()), "World is null at edit command").getBlockAt(ls.getX(), ls.getY(), ls.getZ()).getState();
                    if (!(bs instanceof Sign)) {
                        Bukkit.getLogger().severe("[V10Lift] Wrong sign removed at: " + LocationSerializer.serialize(bs.getBlock().getLocation()));
                        liter.remove();
                        continue;
                    }
                    sign = (Sign) bs;
                    sign.setLine(3, ls.getOldText());
                    sign.update();
                    ls.setOldText(null);
                }
                ConfigUtil.sendMessage(sender, "Edit.TurnedOff");
            } else {
                ConfigUtil.sendMessage(sender, "Edit.StillInEditMode");
                return true;
            }
        } else {
            //TURN ON
            if (args.length < 2) {
                ConfigUtil.sendMessage(sender, "General.IncorrectUsage", Collections.singletonMap("%Command%", "/v10lift edit <Name>"));
                return true;
            }

            if (!DataManager.containsLift(args[1])) {
                ConfigUtil.sendMessage(sender, "General.DoesntExists");
                return true;
            }

            Lift lift = DataManager.getLift(args[1]);
            if (!lift.getOwners().contains(p.getUniqueId()) && !p.hasPermission("v10lift.admin")) {
                ConfigUtil.sendMessage(sender, "General.NoPermission");
                return true;
            }

            DataManager.addEditPlayer(p.getUniqueId(), args[1]);
            BlockState bs;
            Sign sign;
            for (LiftBlock lb : lift.getBlocks()) {
                bs = Objects.requireNonNull(Bukkit.getWorld(lb.getWorld()), "World is null at edit command").getBlockAt(lb.getX(), lb.getY(), lb.getZ()).getState();
                if (!(bs instanceof Sign)) continue;
                sign = (Sign) bs;
                if (!sign.getLine(0).equalsIgnoreCase(ConfigUtil.getConfigText("SignText"))) continue;
                sign.setLine(3, ConfigUtil.getConfigText("MaintenanceText"));
                sign.update();
            }

            Iterator<LiftSign> liter = lift.getSigns().iterator();
            while (liter.hasNext()) {
                LiftSign ls = liter.next();
                bs = Objects.requireNonNull(Bukkit.getWorld(ls.getWorld()), "World is null at edit command").getBlockAt(ls.getX(), ls.getY(), ls.getZ()).getState();
                if (!(bs instanceof Sign)) {
                    Bukkit.getLogger().severe("[V10Lift] Wrong sign removed at: " + LocationSerializer.serialize(bs.getBlock().getLocation()));
                    liter.remove();
                    continue;
                }
                sign = (Sign) bs;
                ls.setOldText(sign.getLine(3));
                sign.setLine(3, ConfigUtil.getConfigText("MaintenanceText"));
                sign.update();
            }
            ConfigUtil.sendMessage(sender, "Edit.TurnedOn");
        }
        return true;
    }
}
