package tech.sbdevelopment.v10lift.locale;

import co.aikar.commands.MessageType;
import co.aikar.locales.MessageKey;
import net.md_5.bungee.api.ChatColor;
import tech.sbdevelopment.v10lift.managers.ACFHandler;

import java.util.Map;

public class Locale {
    private Locale() {
    }

    /**
     * Get a translated message
     *
     * @param key the key to the message
     * @return the translated message string
     */
    public static String getMessage(Message key) {
        return getMessage(key.getMessageKey());
    }

    /**
     * Get a translated message
     *
     * @param msg the key to the message
     * @return the translated message string
     */
    public static String getMessage(MessageKey msg) {
        String message = ACFHandler.getManager().formatMessage(null, MessageType.INFO, msg);
        return ChatColor.translateAlternateColorCodes('&', message);
    }

    /**
     * Get a translated message
     *
     * @param key  the key to the message
     * @param args the arguments to the message
     * @return the translated message string
     */
    public static String getMessage(Message key, Map<String, String> args) {
        return getMessage(key.getMessageKey(), args);
    }

    /**
     * Get a translated message
     *
     * @param msg  the key to the message
     * @param args the arguments to the message
     * @return the translated message string
     */
    public static String getMessage(MessageKey msg, Map<String, String> args) {
        //Turn map into array (key, value, key, value, etc.)
        String[] argsArray = new String[args.size() * 2];
        int i = 0;
        for (Map.Entry<String, String> entry : args.entrySet()) {
            argsArray[i] = entry.getKey();
            argsArray[i + 1] = entry.getValue();
            i += 2;
        }

        String message = ACFHandler.getManager().formatMessage(null, MessageType.INFO, msg, argsArray);
        return ChatColor.translateAlternateColorCodes('&', message);
    }
}