package tech.sbdevelopment.v10lift.managers;

import co.aikar.commands.PaperCommandManager;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.plugin.java.JavaPlugin;
import tech.sbdevelopment.v10lift.V10LiftPlugin;
import tech.sbdevelopment.v10lift.api.objects.Lift;
import tech.sbdevelopment.v10lift.commands.V10LiftCommand;
import tech.sbdevelopment.v10lift.sbutils.YamlFile;

import java.io.File;
import java.io.IOException;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

public class ACFHandler {
    private static final String[] defaultLanguages = new String[]{"en"};
    private static final Pattern pattern = Pattern.compile("^lang_([a-z]{2})\\.yml$");

    private ACFHandler() {
    }

    private static PaperCommandManager manager;

    public static void init(JavaPlugin plugin) throws IOException {
        manager = new PaperCommandManager(plugin);

        manager.getCommandContexts().registerContext(Lift.class, c -> V10LiftPlugin.getApi().getLift(c.popFirstArg().toLowerCase()).orElse(null));

        manager.getCommandCompletions().registerCompletion("lifts", c -> V10LiftPlugin.getApi().getLifts().stream().map(Lift::getName).collect(Collectors.toList()));
        manager.registerCommand(new V10LiftCommand());
        manager.enableUnstableAPI("help");

        manager.getLocales().setDefaultLocale(Locale.forLanguageTag(V10LiftPlugin.getSConfig().getFile().getString("Locale")));

        File localeDir = new File(plugin.getDataFolder(), "locale");
        localeDir.mkdirs();

        for (String lang : defaultLanguages) {
            YamlFile file = new YamlFile("locale/lang_" + lang);
            file.loadDefaults();
        }

        for (File file : localeDir.listFiles((dir, name) -> name.startsWith("lang_") && name.endsWith(".yml"))) {
            Matcher matcher = pattern.matcher(file.getName());
            if (!matcher.matches()) continue;

            try {
                manager.getLocales().loadYamlLanguageFile(file, Locale.forLanguageTag(matcher.group(1)));
            } catch (IOException | InvalidConfigurationException e) {
                plugin.getLogger().severe("Couldn't load the language file " + file.getName() + "!");
            }
        }
    }

    public static PaperCommandManager getManager() {
        return manager;
    }
}
