package tech.sbdevelopment.v10lift.managers;

import org.bukkit.plugin.java.JavaPlugin;
import tech.sbdevelopment.v10lift.V10LiftPlugin;
import tech.sbdevelopment.v10lift.api.objects.Lift;
import tech.sbdevelopment.v10lift.sbutils.StormSQLiteDB;

import java.sql.SQLException;
import java.util.Collection;

/**
 * The DBManager manages the database
 */
public class DBManager {
    private final StormSQLiteDB data;

    /**
     * Construct the database manager
     *
     * @param plugin The plugin instance
     * @param dbName The name of the sqlite database file
     */
    public DBManager(JavaPlugin plugin, String dbName) {
        try {
            data = new StormSQLiteDB(plugin, dbName);
        } catch (SQLException e) {
            throw new RuntimeException("Couldn't connect to the database!", e);
        }
    }

    /**
     * Load the database from data
     *
     * @throws Exception If the data couldn't be loaded
     */
    public void load() throws Exception {
        Collection<Lift> lifts = data.getStorm().buildQuery(Lift.class)
                .execute()
                .join();
        V10LiftPlugin.getApi().getLifts().addAll(lifts);
    }

    /**
     * Save the data to the database
     *
     * @throws SQLException If the data couldn't be saved
     */
    public void save() throws SQLException {
        for (Lift lift : V10LiftPlugin.getApi().getLifts()) {
            data.getStorm().save(lift);
        }
    }

    /**
     * Remove a lift from the database
     *
     * @param lift The lift to remove
     * @throws SQLException If the lift couldn't be removed
     */
    public void remove(Lift lift) throws SQLException {
        data.getStorm().delete(lift);
    }

    /**
     * Close the connection with the database
     */
    public void closeConnection() {
        data.closeSource();
    }
}
